package game;

import game.config.GameUpdate;

import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.swing.JFrame;

//import com.sun.java.swing.plaf.windows.WindowsBorders;

public class GameRunner extends InputAdapter {

    private boolean updating;
    private final Timer timer;
    private final long startTime;
    private long updateTime;
    protected final Game game;
    protected Graphics g;
    protected JFrame window;
    private Vector<KeyEvent> keyEvents;
    private Vector<MouseEvent> mouseEvents;

    public GameRunner(Game game) {

        this.timer = new Timer();
        this.game = game;
        this.startTime = System.currentTimeMillis();
        this.updateTime = startTime;
        this.keyEvents = new Vector<KeyEvent>();
        this.mouseEvents = new Vector<MouseEvent>();

        initWindow();
    }

    protected void initWindow() {

        window = new JFrame(game.getConfig().getTitle());

        window.setSize(game.getConfig().getWindowWidth(), game.getConfig()
                .getWindowHeight());
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setResizable(false);
        window.addKeyListener(this);
        window.addMouseListener(this);
        window.addMouseMotionListener(this);
        window.addMouseWheelListener(this);
        window.setVisible(true);
        
        g = window.getGraphics();
    }

    @Override
    public void keyPressed(KeyEvent e) {
        keyEvents.add(e);
    }

    @Override
    public void keyReleased(KeyEvent arg0) {
        keyEvents.add(arg0);
    }

    @Override
    public void keyTyped(KeyEvent e) {
        keyEvents.add(e);
    }

    @Override
    public void mouseClicked(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mouseDragged(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mouseEntered(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mouseExited(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mouseMoved(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mousePressed(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mouseReleased(MouseEvent arg0) {
        mouseEvents.add(arg0);
    }

    @Override
    public void mouseWheelMoved(MouseWheelEvent arg0) {
        mouseEvents.add(arg0);
    }

    public final void start() {

        window.setVisible(true);
        timer.schedule(new TimerTask() {

            @Override
            public void run() {

                if (!updating) {

                    updating = true;

                    ArrayList<KeyEvent> keyEvents;
                    ArrayList<MouseEvent> mouseEvents;
                    GameUpdate update;
                    long currentTime;

                    keyEvents = new ArrayList<KeyEvent>(
                            GameRunner.this.keyEvents);
                    mouseEvents = new ArrayList<MouseEvent>(
                            GameRunner.this.mouseEvents);
                    GameRunner.this.keyEvents.clear();
                    GameRunner.this.mouseEvents.clear();

                    currentTime = System.currentTimeMillis();
                    updateTime = currentTime;

                    update = new GameUpdate(currentTime - startTime,
                            currentTime - updateTime, keyEvents, mouseEvents);

                    game.update(update);
                    game.draw(g);

                    updating = false;
                }
            }
        }, 0, 17);
    }
}
